//====================================================================================
// Persistence of Vision Ray Tracer Scene Description File
// File: Wind_pattern.pov
// Vers: 3.7
// Desc: Utility for creating a wind pattern effect through rushes.
// Date: September 2014
// Auth: Thomas de Groot
//====================================================================================

#version 3.7;

#include "colors.inc"
#include "rand.inc"
#include "functions.inc"
#include "transforms.inc"
#include "math.inc"

// First you need to create an image map that will be used as basis for a height_field.
// Set Mapping to on to do this. Once you have a satisfactory map, set Mapping to off.


//====================================================================================
// (1) Create an image map
//====================================================================================
// Render settings (right-click on a line below):
// +w640 +h480 +a0.1 +am2 +bm2 +wt6
// +w1024 +h1024 +a0.1 +am2 +bm2 +wt6 +ft

//use +ft for a tga file

global_settings {
        assumed_gamma 1
        //hf_gray_16
        number_of_waves 1
}

camera {
        orthographic
        location <0,0,-10>
        look_at 0
}

#declare R = seed (19);
#declare Pig =
pigment {
        ripples
        color_map {
                [0      srgb <0.01,0.01,0.01>]
                [1      srgb <1,1,1>]
        }
        scale 1/50
        warp {
                turbulence <0.1,0.2,0>*5
                //octaves 9
                //lambda 1.5
                //omega 0.8
        }
        scale <1,4,1>
        rotate 35*z
        scale 50
        frequency 0.1
        poly_wave 1.5
        scale 1
}

#declare Dist = 5;

plane { 
        z, 0
        pigment {
                average
                pigment_map {
                        [1.5             Pig]
                        [RRand (0.5,1,R) Pig scale RRand (0.8, 1.2, R) translate <RRand (-Dist, Dist, R), RRand (-Dist, Dist, R), 0>]
                        [RRand (0.5,1,R) Pig scale RRand (0.8, 1.2, R) translate <RRand (-Dist, Dist, R), RRand (-Dist, Dist, R), 0>]
                        [RRand (0.5,1,R) Pig scale RRand (0.8, 1.2, R) translate <RRand (-Dist, Dist, R), RRand (-Dist, Dist, R), 0>]
                        [RRand (0.5,1,R) Pig scale RRand (0.8, 1.2, R) translate <RRand (-Dist, Dist, R), RRand (-Dist, Dist, R), 0>]
                        [RRand (0.5,1,R) Pig scale RRand (0.8, 1.2, R) translate <RRand (-Dist, Dist, R), RRand (-Dist, Dist, R), 0>]
                }
                scale 0.1
        }
        finish {emission 10}
}