/*
   ----------------               neighbors.pov                 ------------------------
   ----------------             16. February 2016               ------------------------
   ----------------       copyright by Norbert W. Kern          ------------------------
   ----------------      norbert-werner.kern@t-online.de        ------------------------ 
*/

// First you need to run Even more fun with height_fields.pov and Wind_pattern_.pov to get the heightfields (used very high resolutions).

#version 3.7;

#declare RAD = 2;   // used 8 for final render
                
global_settings {
        assumed_gamma 1
        max_trace_level 4
        noise_generator 2  
        #if(RAD > 0)
                radiosity {
                        pretrace_start 0.08
                        pretrace_end   0.04/RAD
                        count 60*RAD
                        nearest_count min (20,RAD)
                        error_bound 2/RAD
                        low_error_factor 0.5
                        recursion_limit 1
                        gray_threshold 0
                        minimum_reuse 0.015
                        brightness 1
                        adc_bailout 0.005
                        normal on
                        media off
                }
        #end
}

#include "stdinc.inc"
#include "functions.inc"

#declare camloc = <-8,3,3>;
#declare lookat = <-2,2.5,-1>;
#declare camang = 55; 

camera { 
        location camloc-(<0,10,2>+<0,6.5,0>)*0.5
        look_at lookat-(<0,10,2>+<0,5.9,0>)*0.5 
        right -x*image_width/image_height 
        up y 
        angle camang
}

//________________________________________________________________________________________________________________________________________________________________ 

sky_sphere {
	pigment {
		gradient y 
		color_map {    
			 [0      srgb 0.1*<0.6,0.85,1>-<120,114,88>/255+<54,76,117>/255+<0,0.1,0.1>]
			 [0.8    srgb 0.4*<0.6,0.85,1>-<120,114,88>/255*0.5+<54,76,117>/255*0.5+<0,0.1,0.1>]
			 [0.87   srgb (0.4*<0.6,0.85,1>+<1,0.9,0.75>)*0.5-<120,114,88>/255*0.13+<54,76,117>/255*0.13+<0,0.1,0.1>*0.5]
			 [0.95   srgb <1,0.9,0.75>-<120,114,88>/255*0.05+<54,76,117>/255*0.05]       
			 [1      srgb 1.7*<1,0.9,0.75>]      
		}
		scale 2 
		translate -y
	}
	rotate -50*z 
	rotate 100*y 
}

light_source {
        <0,1000,0> 
        srgb <2.42,2.23,1.87>*0.5
        area_light <80,0,0>, <0,0,80> 5,5 adaptive 0 jitter circular orient
	rotate -50*z 
	rotate 100*y
}

fog {
        distance 750/0.35
        color srgb <120,114,88>/150
        fog_type 2
        fog_offset 25
        fog_alt 1*4
        turbulence 0.1
        turb_depth 0.2
}

//________________________________________________________________________________________________________________________________________________________________ 

#declare Field =
height_field {
        png "Even more fun with height_fields.png"
        hollow double_illuminate
        translate <-0.1,-0.5,-0.9> 
        scale <400,36,300>
        translate y*-10
}

// all textures are variations of Tekno Frannansa   Porous Rock  http://news.povray.org/povray.binaries.images/message/%3C486e82e5%40news.povray.org%3E/#%3C486e82e5%40news.povray.org%3E

#declare radio = 1;

#declare sponge_pig =
pigment {
        #local Curve    = 0.125; // small values make it look porous
        #local Omega    = 0.5;
        #local Lambda   = 2;
        #local Octaves  = 8;
        pigment_pattern {
                average
                pigment_map {
                        #local octave = 0;
                        #while (octave < Octaves)
                                [pow (Omega, octave)
                                        pigment_pattern {
                                                crackle form <0.9,0,0>
                                                scale pow (Lambda, -octave)
                                                color_map {
                                                        [0      srgb 1]
                                                        [1      srgb 0]
                                                }
                                        }
                                        poly_wave 1/Curve
                                ]
                                #local octave = octave+1;
                        #end
                }
        }
        poly_wave Curve
        color_map {
                [0      srgb <255,243,191>/255]
                [1      srgb 0]
        }
}

#declare T1 =
material {
	texture {
                pigment {sponge_pig}
                normal {
                        pigment_pattern {sponge_pig} 
                        -3 
                        accuracy 0.001 
                        cubic_wave
                }
                finish {
                        ambient 0
                        diffuse albedo 1.4
                        specular albedo 0.07 
                        roughness 0.03
                        brilliance 0.5
                }
        }
	interior {ior 1.5}
	scale 1
}

union {
        object {Field}
        material {T1 scale 3.5}
}

//________________________________________________________________________________________________________________________________________________________________ 

#declare pos1 = lookat-(camloc-lookat)*11+<-23,-12,0>; // sphere left
#declare pos3 = lookat-(camloc-lookat)*11+<187,-8,0>;  // sphere right

#include "wind_pattern.inc"

//________________________________________________________________________________________________________________________________________________________________ 

// Tsutomu Higo  lpt3bfel

#declare Rndm = seed (29);
#declare R1 = seed (5);
#declare Rdg = 5; //radius of arranging sphere

//initialize the coordinates of beads (X = Cp [I][0] Y = Cp [I][1])
#declare Imax = 100;
#declare Cd = array [Imax][3]
#declare I = 0; 
#while (I < Imax)
        #declare J =0 ; 
        #while (J < 3)
                #declare Cd [I][J] = 0;
                #declare J = J+1; 
        #end
        #declare I = I+1; 
#end

//from cos to angle (0-2*pi)
#macro Acosc (Rc, Dy, Ac)

#declare Ac = acos (Rc);
#if (Dy < 0) #declare Ac = 2*pi-Ac; #end
#if (Ac = 0) #declare Ac = 2*pi; #end

#end

//rotate coordinates Nrot at angle Arot
#macro Rota (Nrot, Arot) 

#declare Cdx = Cd [Nrot][0];
#declare Cdy = Cd [Nrot][1];
#declare Cd [Nrot][0] = Cdx*cos (Arot)-Cdy*sin (Arot);
#declare Cd [Nrot][1] = Cdx*sin (Arot)+Cdy*cos (Arot);
  
#end

//coordinates of starting 7 beads (arranged bead radius = 1)
#declare I = 0; 
#while (I < 6)
        #declare Pe = 1+I;
        #declare Cd [Pe][0] = Cd [0][0]+2*cos (pi/3*I);
        #declare Cd [Pe][1] = Cd [0][1]+2*sin (pi/3*I);
        #declare I = I+1; 
#end

//initial declarement
#declare Ap = 0;
#declare As = 0;
#declare Al = 0;
#declare Ae = 0;
#declare An = 0;
#declare Ad = 0;

#macro Spculc (N, P, Pe)

//reduction ratio from plane to arranging shape at point P
#declare Ao = (sqrt (Cd [P][0]*Cd [P][0]+Cd [P][1]*Cd [P][1]))/2/Rdg;
#declare Ro = sin (Ao)*cos (Ao)/Ao;
//rotate to X axis  

Acosc (Cd [N][0]/sqrt (Cd [N][0]*Cd [N][0]+Cd [N][1]*Cd [N][1]),Cd [N][1],An)
Rota (N,-An)
Rota (N+1,-An)  

#declare Cd [N+1][1] = Cd [N+1][1]*Ro; 

Rota (P,-An) 
  
#declare Cd [P][1] = Cd [P][1]*Ro;
#declare Rs = sqrt (pow (Cd [P][0]-Cd [N][0],2)+pow (Cd [P][1]-Cd [N][1],2))/4; 

Acosc ((Cd [P][0]-Cd [N][0])/4/Rs,Cd [P][1]-Cd [N][1],Ap)
#declare Re = sqrt (pow (Cd [N+1][0]-Cd [N][0],2)+pow (Cd [N+1][1]-Cd [N][1],2))/4; 

Acosc ((Cd [N+1][0]-Cd [N][0])/4/Re,Cd [N+1][1]-Cd [N][1],Ae)
Acosc (Rs,1,As)
Acosc (Re,1,Al)

#if (rand (Rndm) > 0.3) 
        #declare Ad = 0.5*rand(Rndm); 
#end //confuse the arrangement

#if (Ae-Ap < 0) 
        #declare Ap = Ap-2*pi; 
#end 

#declare Bo = acos (1/Rdg/tan(pi-2*Ao)); 
#if (Ao > pi/4) 
        #declare Rb = 2*Bo/pi; 
#else 
        #declare Rb = 1; 
#end          

#if (Ae-Ap-As-Al-Ad > 0)
        #declare I = 0; 
        #while (I < int ((Ae-Ap-As-Al-Ad)*Rb*3/pi)+1)
                #declare Pe = P+1+I;
                #declare Cd [Pe][0] = Cd [N][0]+2*cos (Ap+As+Ad+pi/3*I/Rb);
                #declare Cd [Pe][1] = Cd [N][1]+2*sin (Ap+As+Ad+pi/3*I/Rb)/Ro;
                Rota (Pe,An)
                #declare I = I+1; 
        #end
        #declare Ad = 0;
#else
        #if (Ae-Ap-As-Al > 0)
                #declare I = 0; 
                #while (I < int ((Ae-Ap-As-Al)*Rb*3/pi)+1)
                        #declare Pe = P+1+I;
                        #declare Cd [Pe][0] = Cd [N][0]+2*cos (Ap+As+pi/3*I/Rb);
                        #declare Cd [Pe][1] = Cd [N][1]+2*sin (Ap+As+pi/3*I/Rb)/Ro;
                        Rota (Pe,An)
                        #declare I = I+1;
                #end
        #end
#end  

//rotate to former position
Rota (N,An)

#declare Cd [N+1][1] = Cd [N+1][1]/Ro;

Rota (N+1,An)  

#declare Cd [P][1] = Cd [P][1]/Ro; 
    
Rota (P,An)
  
#end

#declare Ps = 6;
#declare Jdl = Imax-int (Rdg*pi)*2;
#declare Jd = 1;  

#while (Jd < Jdl)
        Spculc (Jd,Ps,Pe)
        #declare Ps = Pe;
        #if (Ao > pi/2-1/Rdg) 
                #declare Jd = Jdl;
        #end
        #declare Jd = Jd+1; 
#end

#declare I = 0;  

#while (I < Pe-1)
        //reduction ratio from plane to arranging shape at point P
        #if (I > 0)
                #declare Ai = sqrt (Cd [I][0]*Cd [I][0]+Cd [I][1]*Cd [I][1])/2/Rdg;
                #declare Ri = sin (Ai)*cos (Ai)/Ai;
        #else
                #declare Ai = 0;
                #declare Ri = 1;
        #end
        //arranging beads and calculating the coordinates of the holes position
        #declare Cd [I][0] = Ri*Cd [I][0];
        #declare Cd [I][1] = Ri*Cd [I][1];
        #declare Cd [I][2] = Rdg-2*Rdg*sin (Ai)*sin (Ai);
        #declare I = I+1; 
#end
             
//initialize the coordinates for mesh Cp(x,y,z,r)
#declare Vmax = 72; //
#declare Hmax = 2*Vmax;
#declare Cp = array [Hmax][Vmax+2][4]
#declare V = 0;
 
#while (V < Vmax+2)
        #declare H = 0; 
        #while (H < Hmax)
                #declare I = 0; 
                #while (I<4)
                        #declare Cp [H][V][I] = 0;
                        #declare I = I+1; 
                #end
                #declare H = H+1; 
        #end
        #declare V = V+1; 
#end

//triangle of the surface

#macro Trid (Ph, Pv)

#declare Ph1 = mod (Ph-1+Hmax,Hmax);
triangle {
        <Cp [Ph][Pv][0],Cp [Ph][Pv][1],Cp [Ph][Pv][2]>,
        <Cp [Ph1][Pv-1][0],Cp [Ph1][Pv-1][1],Cp [Ph1][Pv-1][2]>,
        <Cp [Ph][Pv-1][0],Cp [Ph][Pv-1][1],Cp [Ph][Pv-1][2]>
}
triangle {
        <Cp [Ph1][Pv][0],Cp [Ph1][Pv][1],Cp [Ph1][Pv][2]>,
        <Cp [Ph1][Pv-1][0],Cp [Ph1][Pv-1][1],Cp [Ph1][Pv-1][2]>,
        <Cp [Ph][Pv][0],Cp [Ph][Pv][1],Cp [Ph][Pv][2]>
}
    
#end

//normal vector of the plar coordinates point
#macro Vd (Ph, Pv)

<(Cp [mod (Ph-1+Hmax,Hmax)][Pv][1]-Cp [Ph][Pv][1])*(Cp [Ph][Pv-1][2]-Cp [Ph][Pv][2])-(Cp [Ph][Pv-1][1]-Cp [Ph][Pv][1])*(Cp [mod (Ph-1+Hmax,Hmax)][Pv][2]-Cp [Ph][Pv][2]),
(Cp [Ph][Pv-1][0]-Cp [Ph][Pv][0])*(Cp [mod (Ph-1+Hmax,Hmax)][Pv][2]-Cp [Ph][Pv][2])-(Cp [mod (Ph-1+Hmax,Hmax)][Pv][0]-Cp [Ph][Pv][0])*(Cp [Ph][Pv-1][2]-Cp [Ph][Pv][2]),
(Cp [mod (Ph-1+Hmax,Hmax)][Pv][0]-Cp [Ph][Pv][0])*(Cp [Ph][Pv-1][1]-Cp [Ph][Pv][1])-(Cp [Ph][Pv-1][0]-Cp [Ph][Pv][0])*(Cp [mod (Ph-1+Hmax,Hmax)][Pv][1]-Cp [Ph][Pv][1])> 

#end

//smooth_triangle of the surface
#macro Trisd (Ph, Pv)

#declare Ph1 = mod (Ph-1+Hmax,
Hmax);
smooth_triangle { 
         <Cp [Ph][Pv][0],Cp [Ph][Pv][1],Cp [Ph][Pv][2]>, Vd (Ph,Pv),
         <Cp [Ph1][Pv-1][0],Cp [Ph1][Pv-1][1],Cp [Ph1][Pv-1][2]>, Vd (Ph1,Pv-1),
         <Cp [Ph][Pv-1][0],Cp [Ph][Pv-1][1],Cp [Ph][Pv-1][2]>, Vd (Ph,Pv-1)
}
smooth_triangle { 
         <Cp [Ph1][Pv][0],Cp [Ph1][Pv][1],Cp [Ph1][Pv][2]>, Vd (Ph1,Pv),
         <Cp [Ph1][Pv-1][0],Cp [Ph1][Pv-1][1],Cp [Ph1][Pv-1][2]>, Vd (Ph1,Pv-1),
         <Cp [Ph][Pv][0],Cp [Ph][Pv][1],Cp [Ph][Pv][2]>, Vd (Ph,Pv)
}
          
#end

//radius caliculation and the translation from polar to xyz coordinsates
#declare V = 0; 
#while (V < Vmax+2)
        #declare H = 0; 
        #while (H < Hmax)  
                #if (V = 0) 
                        #declare Vvmax = 0; 
                #else 
                        #declare Vvmax = 0.999998*(V-1)/Vmax+0.000001; 
                #end
                #declare Cp [H][V][3] = Rdg; //radius
                #declare Cp [H][V][0] = Cp [H][V][3]*sin (pi*Vvmax)*cos (2*pi*H/Hmax);
                #declare Cp [H][V][1] = Cp [H][V][3]*sin (pi*Vvmax)*sin (2*pi*H/Hmax);
                #declare Cp [H][V][2] = -Cp [H][V][3]*cos (pi*Vvmax);
                #declare P = 0; 
                #while (P < Pe)
                        #declare Cd2 = pow (Cp [H][V][0]-Cd [P][0],2)+pow (Cp [H][V][1]-Cd [P][1],2)+pow (Cp [H][V][2]-Cd [P][2],2);
                        #declare Cp [H][V][3] = Cp [H][V][3]-1.6*0.1/(Cd2+0.1);
                        #declare P = P+1; 
                #end
                #declare Cp [H][V][0] = Cp [H][V][3]*sin (pi*Vvmax)*cos (2*pi*H/Hmax);
                #declare Cp [H][V][1] = Cp [H][V][3]*sin (pi*Vvmax)*sin (2*pi*H/Hmax);
                #declare Cp [H][V][2] = -Cp [H][V][3]*cos (pi*Vvmax);
                #declare H = H+1;
        #end
        #declare V = V+1; 
#end

//________________________________________________________________________________________________________________________________________________________________ 

#declare bz = pigment {bozo scale 0.3}
#declare m0 = 0;  
#declare stp = 0.08; 
#declare s1 = stp*0.5;
#declare col = srgb <1,0.7,0.3>;
#declare stri =
color_map {
        #while (m0 < 1) 
                [m0     col] 
                [m0+s1  0.15*col]
                #declare m0 = m0+stp;  
        #end
}

#undef FN
#declare FN = 
function { 
        pigment {
                bz 
                color_map {stri}
                scale 7
        }
}

#declare sponge_pig =
pigment {
        #local Curve    = 0.125; // small values make it look porous
        #local Omega    = 0.5;
        #local Lambda   = 2;
        #local Octaves  = 8;
        pigment_pattern {
                average
                pigment_map {
                        #local octave = 0;
                        #while (octave < Octaves)
                                [pow (Omega, octave)
                                        pigment_pattern {
                                                function {FN (x,y,z).x}
                                                scale pow (Lambda, -octave)
                                                color_map {
                                                        [0      srgb 1]
                                                        [1      srgb 0]
                                                }
                                        }
                                        poly_wave 1/Curve
                                ]
                                #local octave = octave+1;
                        #end
                }
        }
        poly_wave Curve
        color_map { 
                [0      srgb <0,0.05,0.05>]
                [0.25   srgb <0.15,0.1,0.15>]
                [0.75   srgb <0.8,0.7,0.5>]
                [1      srgb 1]
        }
}

#declare T2 =
material {
	texture {
                pigment {sponge_pig}
                normal {
                        pigment_pattern {sponge_pig} 
                        -2 
                        accuracy 0.001 
                        cubic_wave
                }
                finish {
                        ambient 0
                        diffuse albedo 0.26
                        specular albedo 0.3 
                        roughness 0.001
                        reflection {0.3}
                }
        }
	interior {ior 1.5}
	scale 1
}
 
//________________________________________________________________________________________________________________________________________________________________ 

#declare sponge_pig2 =
pigment {
        #local Curve    = 0.125; // small values make it look porous
        #local Omega    = 0.5;
        #local Lambda   = 2;
        #local Octaves  = 8;
        pigment_pattern {
                average
                pigment_map {
                        #local octave = 0;
                        #while (octave < Octaves)
                                [pow (Omega, octave)
                                        pigment_pattern {
                                                function {FN (x,y,z).x}
                                                scale pow (Lambda, -octave)
                                                color_map {
                                                        [0      srgb 1]
                                                        [1      srgb 0]
                                                }
                                        }
                                        poly_wave 1/Curve
                                ]
                                #local octave = octave+1;
                        #end
                }
        }
        poly_wave Curve
        color_map {
                [0      srgb <0,0,0.05>]
                [0.25   srgb <0.1,0.15,0.15>]
                [0.75   srgb <0.3,0.4,0.5>]
                [1      srgb 1]
        }
}

#declare T3 =
material {
	texture {
                pigment {sponge_pig2}
                normal {
                        pigment_pattern {sponge_pig2} 
                        -2 
                        accuracy 0.001 
                        cubic_wave
                }
                finish {
                        ambient 0
                        diffuse albedo 0.26
                        specular albedo 0.3 
                        roughness 0.001
                        reflection {0.3}
                }
        }
	interior {ior 1.5}
	scale 1
}

//________________________________________________________________________________________________________________________________________________________________ 

declare meshsphere =       
union {
        mesh {
                #declare V = 2; 
                #while (V < Vmax+2)
                        #declare H = 0; 
                        #while (H < Hmax)
                                Trisd (H, V)
                                #declare H  =H+1; 
                        #end
                        #declare V = V+1; 
                #end
        }      
        material {T3}
        hollow double_illuminate
}

union {
        meshsphere 
        rotate <30,20,60>
        rotate -40*y 
        scale 4.4
        translate pos3
}

//________________________________________________________________________________________________________________________________________________________________ 

// Paul Bourke  Supershape fractal  http://paulbourke.net/geometry/supershape/0003.html

#include "param.inc"

#macro SuperShape (SS1m, SS1a, SS1b, SS1n1, SS1n2, SS1n3, SS2m, SS2a, SS2b, SS2n1, SS2n2, SS2n3, resol) 

#declare SS1 = 
function (T) { 
        1/pow (pow (abs (cos (SS1m*T/4))/SS1a,SS1n2) + pow (abs (sin (SS1m*T/4))/SS1b,SS1n3), 1/SS1n1)
}
#declare SS2 = 
function (T) { 
        1/pow (pow (abs (cos (SS2m*T/4))/SS2a,SS2n2) + pow (abs (sin (SS2m*T/4))/SS2b,SS2n3), 1/SS2n1)
}

#declare Fx = function (u,v) {SS1 (u)*cos (u)*SS2 (v)*cos (v)}
#declare Fy = function (u,v) {SS1 (u)*sin (u)*SS2 (v)*cos (v)}
#declare Fz = function (u,v) {SS2 (v)*sin (v)}

Parametric (Fx, Fy, Fz, <-pi/2,-pi>, <pi/2,pi>, resol, resol, "")

#end

#declare bowl = 
union {
        object {
                SuperShape (8, 1, 2, 20, -1, -1, 8, 1, 2, 20, -1, -1, 800)  
                scale 1.111*4
                rotate -1*<30,20,60> 
        }
        #undef SS1
        #undef SS2
        #undef Fx
        #undef Fy
        #undef Fz 
        object {
                SuperShape (8, 1, 2, 20, -1, -1, 8, 1, 2, 20, -1, -1, 800)  
                scale 1.111*4
        }
        material {T2}
        hollow double_illuminate 
        scale 3.75*0.9
        translate lookat-(camloc-lookat)*11+<-23,-6,0>
}

object {bowl}
