// based on this source: Thomas de Groot  Wind pattern code and reed stalk example  http://news.povray.org/povray.binaries.scene-files/thread/%3C541c256b%40news.povray.org%3E/?ttop=420065&toff=50

//====================================================================================
// Persistence of Vision Ray Tracer Scene Description File
// File: Wind_pattern.pov
// Vers: 3.7
// Desc: Utility for creating a wind pattern effect through rushes.
// Date: September 2014
// Auth: Thomas de Groot
//====================================================================================

#declare swi = 30;
#declare swi2 = 36;

//====================================================================================
// Building of the height_field:

#declare HF_scale = <20,2.5,20>;
#declare HF_res   = 2000;

#declare F_HF_01 =
function {
        pigment {
                image_map {
                        png "Wind_pattern_" gamma 1
                        map_type 0
                        interpolate 2
                }
                warp {repeat x}
                warp {repeat y}
                scale 50
                warp {
                        turbulence 0.2 
                        octaves 1 //[6]
                        lambda 1  //[2]
                        omega 0.2 //[0.5]
                }
                scale 1/50
        }
}

#declare hf =
intersection {
        height_field {
                function HF_res, HF_res {F_HF_01 (x,y,z).hf}
                smooth
                translate <-0.5,0,-0.5>
                scale HF_scale
        }
        box {
                <-0.5,0,-0.5>, <0.5,1,0.5>
                scale HF_scale*<0.99,1,0.99>
        }
        //pigment {srgb <0.6,0.6,0.6>}
        translate pos1
}

//object {hf}

//====================================================================================
// An object used for the selection of random locations:

#local Box = box {<-0.5, 100, -0.5>, <0.5, 100.1, 0.5> scale <HF_scale.x*0.98, 1, HF_scale.z*0.98> translate pos1}
#local R = seed (0);

//====================================================================================

#declare decr = 0.1;
#declare radi = 0.6;

#macro part (base, high, n)

union {
        cone {<0,base,0>, radi*pow (1-decr, n-1), <0,high,0>, radi*pow (1-decr, n)}
        #if (n = 12)
                sphere {<0,high,0>, radi*pow (1-decr, n)*1}
        #else
                sphere {<0,high,0>, radi*pow (1-decr, n)}
        #end
}

#end
  
//====================================================================================

#macro Reed (Bend)

//random rotation for the whole stalk:
#local Rot = RRand (-180, 180, R);

//construction of the Inverse kinematic stalk:
#local Bend1 = Bend+((2*Bend)/10); 

#local Stem11 = 
union {
        object {part (10.4, 10.62, 12) double_illuminate hollow translate -10.40*y rotate Rot*y rotate Bend1*x translate (10.40-10.18)*y }
        object {part (10.18, 10.4, 11) double_illuminate hollow translate -10.18*y rotate Rot*y}
}

#local Bend2 = Bend+(Bend/10);
#local Stem10 =
union {
        object {Stem11 rotate Bend2*x translate (10.18-9.86)*y}
        object {part (9.86, 10.18, 10) double_illuminate hollow translate -9.86*y rotate Rot*y}
}

#local Stem9 =
union {
        object {Stem10 rotate Bend*x translate (9.86-9.54)*y}
        object {part (9.54, 9.86, 9) double_illuminate hollow translate -9.54*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem8 =
union {
        object {Stem9 rotate Bend*x translate (9.54-9.165)*y}
        object {part (9.165, 9.54, 8) double_illuminate hollow translate -9.165*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem7 =
union {
        object {Stem8 rotate Bend*x translate (9.165-8.79)*y}
        object {part (8.79, 9.165, 7) double_illuminate hollow translate -8.79*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem6 =
union {
        object {Stem7 rotate Bend*x translate (8.79-7.72)*y}
        object {part (7.72, 8.79, 6) double_illuminate hollow translate -7.72*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem5 =
union {
        object {Stem6 rotate Bend*x translate (7.72-6.20)*y}
        object {part (6.2, 7.72, 5) double_illuminate hollow translate -6.20*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem4 =
union {
        object {Stem5 rotate Bend*x translate (6.20-4.65)*y}
        object {part (4.65, 6.2, 4) double_illuminate hollow translate -4.65*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem3 =
union {
        object {Stem4 rotate Bend*x translate (4.65-3.10)*y}
        object {part (3.1, 4.65, 3) double_illuminate hollow translate -3.10*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#local Stem2 =
union {
        object {Stem3 rotate Bend*x translate (3.10-1.55)*y}
        object {part (1.55, 3.1, 2)  double_illuminate hollow translate -1.55*y rotate Rot*y}
}

#local Bend = Bend-(Bend/20);
#declare ReedMesh_03 =
union {
        object {Stem2 rotate Bend*x translate 1.55*y}
        object {part (0, 1.55, 1) double_illuminate hollow rotate Rot*y}
}

#end  //macro

//====================================================================================
                       
#declare f_obj2  = function (x,y,z) {f_sphere (x,y,z, 0.05)}
#declare FN2 = function (x,y,z) {f_obj2 (2*f_noise3d (x,y,z)-1, 2*f_noise3d (x,y,z)-1, 2*f_noise3d (x,y,z)-1)} 

#declare sponge_pig =
pigment {
        #local Curve    = 0.125; // small values make it look porous
        #local Omega    = 0.5;
        #local Lambda   = 2;
        #local Octaves  = 8;
        pigment_pattern {
                average
                pigment_map {
                        #local octave = 0;
                        #while (octave < Octaves)
                                [pow (Omega, octave)
                                        pigment_pattern {
                                                function {FN2 (x,y,z)}
                                                scale pow (Lambda, -octave)
                                                color_map {
                                                        [0      srgb 1]
                                                        [1      srgb 0]
                                                }
                                        }
                                        poly_wave 1/Curve
                                ]
                                #local octave = octave+1;
                        #end
                }
        }
        poly_wave Curve
        color_map {
                [0      srgb <0.05,0,0>]
                [0.25   srgb <0.15,0.1,0.15>]
                [0.75   srgb <0.8,0.4,0.3>]
                [1      srgb 1]
        }
}

#declare T1 =
material {
	texture {
                pigment {sponge_pig}
                normal {
                        pigment_pattern {sponge_pig} 
                        -4 
                        accuracy 0.001 
                        cubic_wave
                }
                finish {
                        ambient 0
                        diffuse albedo 0.55
                        specular albedo 0.3 
                        roughness 0.001
                        reflection {0.3}
                }
        }
	interior {ior 1.5}
	scale 1
}

//====================================================================================
// Planting the reed stalks:  

#declare pla =
function {
	pigment {
		bumps
		color_map {
			[0    srgb 0]
			[1    srgb 1]
		}
                scale 2.5
	}
}

//wind strength (>0.0 to <=1.0):
#declare WindStrength = 0.65;

#declare xet = seed (2);
#declare yet = seed (16);
#declare zet = seed (17);

#for (N, 0, 3000)       // sphere left
        #local Norm = <0,0,0>;
        #local Norm2 = <0,0,0>;
        #declare fix = -swi+swi*rand (xet)*2+pos1.x;
        #declare fiz = -swi+swi*rand (xet)*2+pos1.z;       
        #declare Start = <fix,1000,fiz>;
        #local Pos = trace (hf, Start, -y, Norm);
        #local Posi = trace (Field, Start, -y, Norm2);
        #if (Norm.x = 0 & Norm.y <= 0 & Norm.z = 0)
                #local Norm = <0.2,1,0.2>;
        #end
        #declare dis = vlength (<pos1.x,0,pos1.z>-<Posi.x,0,Posi.z>);
        #if ((vlength (Norm2) > 0) & (dis > 16))
                #local VerAng = VAngleD (<Norm.x,0,Norm.z>, Norm);
                #local HorAng = VRotationD (<0,0,1>, Norm, y);
                #if (HorAng < 0) #local HorAng = HorAng+180; #end
                Reed ((90-VerAng)*WindStrength*(0.5+rand (yet)))
                object {
                        ReedMesh_03 
                        scale RRand (0.8, 1.2, R)*pla (Posi.x,Posi.y,Posi.z).x*(37-dis)/21 
                        rotate (HorAng-45+(-0.5+rand (yet))*40)*y 
                        Reorient_Trans (y, (Norm2+2*y)) 
                        translate <Posi.x,Posi.y-0.5,Posi.z> 
                        material {T1 scale 0.2 translate 10000*rand (zet)}
                }
        #end
#end 

#declare xeu = seed (14);
#declare yeu = seed (28);
#declare zeu = seed (42);

#for (N, 0, 4500)       // sphere right
        #local Norm = <0,0,0>;
        #local Norm2 = <0,0,0>;
        #declare fix = -swi2+swi2*rand (xeu)*2+pos3.x;
        #declare fiz = -swi2+swi2*rand (xeu)*2+pos3.z;       
        #declare Start = <fix,1000,fiz>;
        #local Pos = trace (hf, Start, -y, Norm);
        #local Posi = trace (Field, Start, -y, Norm2);
        #if (Norm.x = 0 & Norm.y <= 0 & Norm.z = 0)
                #local Norm = <0.2,1,0.2>;
        #end
        #declare dis = vlength (<pos3.x,0,pos3.z>-<Posi.x,0,Posi.z>);
        #if ((vlength (Norm2) > 0) & (dis > 20))
                #local VerAng = VAngleD (<Norm.x,0,Norm.z>, Norm);
                #local HorAng = VRotationD (<0,0,1>, Norm, y);
                #if (HorAng < 0) #local HorAng = HorAng+180; #end
                Reed ((90-VerAng)*WindStrength*(0.5+rand (yeu)))
                object {
                        ReedMesh_03 
                        scale RRand (0.8, 1.2, R)*pla (Posi.x,Posi.y,Posi.z).x*(41-dis)/25 
                        rotate (HorAng-45+(-0.5+rand (yeu))*40)*y 
                        Reorient_Trans (y, (Norm2+2*y)) 
                        translate <Posi.x,Posi.y-0.5,Posi.z> 
                        material {T1 scale 0.2 translate 10000*rand (zeu)}
                }
        #end
#end
 